require 'oj'
require 'typhoeus'
require 'recursive-open-struct'
require 'turba_kit/error'
require 'turba_kit/countries'
require 'turba_kit/checkins'
require 'turba_kit/availability'
require 'turba_kit/units'
require 'turba_kit/services'
require 'turba_kit/crowd_table'
require 'turba_kit/link'
require 'turba_kit/misc'
require 'turba_kit/call_to_action'
require 'turba_kit/phonenumber'
require 'turba_kit/tip'
require 'turba_kit/content'
require 'turba_kit/place'
require 'turba_kit/notification'
require 'active_support/core_ext/hash'

module TurbaKit
  class Client
    include TurbaKit::Countries
    include TurbaKit::Checkins
    include TurbaKit::Availability
    include TurbaKit::Units
    include TurbaKit::Services
    include TurbaKit::CrowdTable
    include TurbaKit::Link
    include TurbaKit::Misc
    include TurbaKit::CallToAction
    include TurbaKit::Phonenumber
    include TurbaKit::Tip
    include TurbaKit::Content
    include TurbaKit::Place
    include TurbaKit::Notification

    def initialize(api_key, options = {})
      @api_key = api_key
      default_options = {
        version: '/v2',
        domain: 'api.turba-webservices.com',
        scheme: 'https'
      }
      @options = default_options.merge(options)
    end

    def api_url
      @api_url ||= "#{@options[:scheme]}://#{@options[:domain]}#{@options[:version]}"
    end

    def default_params
      @default_params ||= { api_key: @api_key }
    end

    def put_request(endpoint, options = {})
      options[:namespace] ||= nil
      options[:accept] ||= "application/json"
      options[:params] ||= {}

      request = Typhoeus::Request.new(
        "#{api_url}#{endpoint}",
        method: :put,
        params: default_params,
        body: Oj.dump(options[:params], mode: :compat),
        headers: { "Content-Type" => options[:accept] }
      ).run
      response(request, options[:namespace])
    end

    def post_request(endpoint, options = {})
      options[:namespace] ||= nil
      options[:accept] ||= "application/json"
      options[:params] ||= {}

      request = Typhoeus::Request.new(
        "#{api_url}#{endpoint}",
        method: :post,
        params: default_params,
        body: Oj.dump(options[:params], mode: :compat),
        headers: { "Content-Type" => options[:accept] }
      ).run
      response(request, options[:namespace])
    end

    def delete_request(endpoint)
      request = Typhoeus::Request.new(
        "#{api_url}#{endpoint}",
        method: :delete,
        params: default_params,
      ).run

      response(request, nil)
    end

    def get_request(endpoint, options = {})
      options[:namespace] ||= nil
      options[:accept] ||= "application/json"
      options[:params] ||= {}

      request = Typhoeus::Request.new(
        "#{api_url}#{endpoint}",
        method: :get,
        params: default_params.merge(options[:params]),
        headers: { Accept: options[:accept] }
      ).run

      p request

      response(request, options[:namespace])
    end

    def response(request, namespace)
      body = Oj.load(request.body)
      body = body.with_indifferent_access unless body.nil?
      status = request.options[:response_code]
      body ||= request.body

      if (400..599).include?(status)
        return Error.from_response(body, status)
      end

      return "" if body.empty?
      RecursiveOpenStruct.new(body, recurse_over_arrays: true)
    end
  end
end
